#include "includes.h"
#include "render.h"
#include "input.h"
#include "tga.h"
#include "config.h"


#define CAM_X 2.0
#define CAM_Y -1.0
#define CAM_Z -1.0

#define SPHERE_SPACING 1.05
#define SPHERE_OFFSET  0.4
#define SPHERE_SELECTED 0.7
#define ICON_OFFSET 0.05
#define GLOW_LOW  0.45
#define GLOW_HIGH 0.6
#define GLOW_DELTA 0.05

GLint button_display_list, glow_display_list, bg_display_list, icon_display_list;
float glow, bgz1, bgz2;
GLuint *texture;
enum Texture_Objects { SPHERE = 0, BG, ICON };
GLUquadricObj* m_pQuadric;


void funcRedraw(void){
	int x, y;
	float s, xx, yy, bb;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// draw background
	bgz1--; bgz2--;
	if(bgz1 < -400.0) bgz1 = 400.0;
	if(bgz2 < -400.0) bgz2 = 400.0; 
	glPushMatrix();
	glTranslatef(0.0, 0.0, bgz1);
	glCallList(bg_display_list);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(0.0, 0.0, bgz2);
	glCallList(bg_display_list);
	glPopMatrix();

	glow += GLOW_DELTA; if(glow > GLOW_HIGH) glow = GLOW_LOW;

	// draw buttons
	for(y = 0; y > -NR_VERT_BUTTONS; y--){
		for(x = 0; x < NR_HORZ_BUTTONS; x++){
			// get random shift value
			s = (rand()/(RAND_MAX*1.0)-0.5)/50;
			xx = x * SPHERE_SPACING + s + SPHERE_OFFSET;
			yy = y * SPHERE_SPACING + s - SPHERE_OFFSET;
			bb = button_base + y*NR_HORZ_BUTTONS-x;
			glPushMatrix();
			glTranslatef(xx, yy, -5.0);

			// draw button
			glCallList(button_display_list);

			// draw glow
			glDisable(GL_TEXTURE_2D);
			if(button == bb) glColor3f(0.0f, SPHERE_SELECTED, glow);
			else glColor3f(0.0f, 0.0f, glow);
			glTranslatef(0.0, 0.0, -glow/4.0);
			glCallList(glow_display_list);
			glEnable(GL_TEXTURE_2D);

			glTranslatef(-(x-NR_HORZ_BUTTONS/2.0)/10.0-ICON_OFFSET,
					-(y+NR_VERT_BUTTONS/2.0)/10.0+ICON_OFFSET,
					0.5);

			// draw icon
			if(bb >= max_buttons){
				glBindTexture(GL_TEXTURE_2D, texture[ICON-(int)bb]);
				glCallList(icon_display_list);
			}

			// draw text


			glPopMatrix();
		}
	}

	glFlush();
	glutSwapBuffers();

/////// uncomment below when used with hardware 3d
//	usleep(200000);
}


//////////////////////////////////////////////////////////////////////


void drawIcon(){
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.3f,  0.3f, 0.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.3f, -0.3f, 0.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 0.3f, -0.3f, 0.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 0.3f,  0.3f, 0.0f);
	glEnd();
}
void drawButton(){
	glTranslatef(0.0f, 0.5f, 0.0f);
	glBindTexture(GL_TEXTURE_2D, texture[SPHERE]);
	gluSphere(m_pQuadric, 0.45f, 20, 20);
}
void drawGlow(){
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glScalef(1.0, 1.0, 1.2);
	gluSphere(m_pQuadric, 0.5f, 20, 20);
	glDisable(GL_BLEND);
}
void drawBackground(){
	glBindTexture(GL_TEXTURE_2D, texture[BG]);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-4.0f, -NR_VERT_BUTTONS*SPHERE_SPACING,    0.0f);
		glTexCoord2f(0.0f, 4.0f); glVertex3f(-4.0f, -NR_VERT_BUTTONS*SPHERE_SPACING, -400.0f);
		glTexCoord2f(4.0f, 4.0f); glVertex3f( 8.0f, -NR_VERT_BUTTONS*SPHERE_SPACING, -400.0f);
		glTexCoord2f(4.0f, 0.0f); glVertex3f( 8.0f, -NR_VERT_BUTTONS*SPHERE_SPACING,    0.0f);
	glEnd();
}


GLuint createDL(void (*func)()){
	GLuint newDL;

	newDL = glGenLists(1);
	glNewList(newDL, GL_COMPILE);
	func();
	glEndList();
	return(newDL);
}


int initTex(char *filename){
	tgaInfo *textureImage;

	textureImage = tgaLoad(filename);
	if(textureImage->status){
		printf("WARNING: unable to load image file %s\n", filename);
		tgaDestroy(textureImage);
		return(0);
	}
printf("TEXTURE: %s (%d-bps)\n", filename, textureImage->pixelDepth);
	glTexImage2D(GL_TEXTURE_2D, 0, (textureImage->pixelDepth)>>3, textureImage->width,
			textureImage->height, 0, textureImage->pixelDepth == 32 ? GL_RGBA : GL_RGB,
			GL_UNSIGNED_BYTE, textureImage->imageData);
	tgaDestroy(textureImage);
	return(1);
}


void initScene(){
	int i;
	struct icon_struct *icon_node;

	texture = (GLuint *)malloc(sizeof(GLuint)*(2+nr_icons));

	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();
	glViewport(0, 0, (GLint)WIDTH, (GLint)HEIGHT);
	gluPerspective(45, 1.0 * WIDTH / HEIGHT, 1, 400);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(CAM_X,CAM_Y,CAM_Z, CAM_X,CAM_Y,CAM_Z-1.0, 0.0,1.0,0.0);

	glGenTextures(2+nr_icons, texture);

	glBindTexture(GL_TEXTURE_2D, texture[SPHERE]);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	if(!initTex("textures/lapis.tga")){
		printf("ERROR: unable to load required texture\n");
		exit(1);
	}

	glBindTexture(GL_TEXTURE_2D, texture[BG]);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	if(!initTex("textures/ether.tga")){
		printf("ERROR: unable to load required texture\n");
		exit(1);
	}

	icon_node = icons;
	for(i = 0; i < nr_icons; i++){
		glBindTexture(GL_TEXTURE_2D, texture[ICON+i]);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		if(!initTex(icon_node->path)){
			printf("WARNING: unable to load icon image %s, no more icons will be loaded\n",
				icon_node->path);
			nr_icons = i;
			break;
		}
		icon_node = icon_node->next;
	}


	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL); 
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	m_pQuadric = gluNewQuadric();
	gluQuadricTexture(m_pQuadric, GL_TRUE);

	button_display_list = createDL(drawButton);
	glow_display_list = createDL(drawGlow);
	bg_display_list = createDL(drawBackground);
	icon_display_list = createDL(drawIcon);

	glow = GLOW_LOW;
	bgz1 = 400.0; bgz2 = 0.0;
}


void shutdown(){
	if(m_pQuadric) gluDeleteQuadric(m_pQuadric);
}


void launch(){
	struct icon_struct *icon_node;
	char app[300];
	int i;

	// fly selected sphere back

	// fly selected sphere forward thru center

	// knock away any spheres the selected sphere collides with

	// explode selected sphere


	// execute application
	icon_node = icons;
	for(i = 0; i > button; i--) icon_node = icon_node->next;
	strcpy(app, icon_node->program);
	strcat(app, " -geometry 640x480+0+0");
printf("EXEC: %s\n", app);  
	system(app);

	// shutdown, menu will be restarted when app exits or hides
	shutdown();
	exit(0);
}
